/*
    Valtra Tractors
    Copyright 2004 Martin Storsj

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Martin Storsj
    martin@martin.st
*/

#ifndef __ZIP_H
#define __ZIP_H

#include <SDL.h>
#include <SDL_endian.h>
#include <string>
#include <vector>

using std::string;
using std::vector;

class ZipEntry {
private:
public:
	string name;
	int offset;
	int compression;
	int csize, dsize;
	ZipEntry(const char *n, int o, int c, int cs, int ds) : name(n), offset(o), compression(c), csize(cs), dsize(ds) {}
};

class FileSource {
public:
	virtual ~FileSource() {}
	virtual SDL_RWops *getFile(const char *n, bool need_end = false) = 0;
	virtual void listFiles(const char *dir, vector<string> &files) = 0;
};

class ZipFile : public FileSource {
private:
	Uint8 *data;
	string name;
	vector<ZipEntry> entries;
	void init(SDL_RWops *file);
	int namecmp(const char *n1, const char *n2);
public:
	ZipFile(void *ptr, int size);
	ZipFile(const char *name);
	~ZipFile();
	SDL_RWops *getFile(const char *n, bool need_end = false);
	void listFiles(const char *dir, vector<string> &files);
};


class RealFileSystem : public FileSource {
private:
	const char *root;
public:
	RealFileSystem();
	RealFileSystem(const char *r);
	~RealFileSystem();
	SDL_RWops *getFile(const char *n, bool need_end = false);
	void listFiles(const char *dir, vector<string> &files);
};

const int FS_REAL = 0x00001;
const int FS_ZIP =  0x00002;
const int FS_RES =  0x00004;

class FileSystem {
private:
	vector<FileSource*> sources;
	void loadRes();
public:
	FileSystem(int srcs = FS_REAL|FS_ZIP|FS_RES);
	~FileSystem();
	SDL_RWops *getFile(const char *n, bool need_end = false);
	void listFiles(const char *dir, vector<string> &files);
};



extern FileSystem fsys;

#define IMG_Load(x) IMG_Load_RW(fsys.getFile(x),1)
#define TTF_OpenFont(x,y) TTF_OpenFontRW(fsys.getFile(x,true),1,y)


#endif
