/*
    Valtra Tractors
    Copyright 2004 Martin Storsj

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Martin Storsj
    martin@martin.st
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <SDL.h>
#include <SDL_image.h>
#include <time.h>
#include "game.h"

//#define FPS

int nUpdated = 0;
SDL_Rect *rects = NULL;
int arraySize = 25;

void updatedRect(const SDL_Rect &r) {
	if (nUpdated==arraySize) {
		int newArraySize = arraySize*2;
		SDL_Rect *newArray = (SDL_Rect*) realloc(rects,sizeof(SDL_Rect)*newArraySize);
		if (newArray==NULL) {
			fprintf(stderr,"Out of memory\n");
			return;
		} else {
			arraySize = newArraySize;
			rects = newArray;
		}
	}
	rects[nUpdated] = r;
	nUpdated++;
}

int main(int argc, char *argv[]) {
	if (SDL_Init(SDL_INIT_VIDEO|SDL_INIT_NOPARACHUTE)<0) {
		fprintf(stderr,"%s\n",SDL_GetError());
		return 1;
	}
	if (atexit(SDL_Quit)) {
		fprintf(stderr,"atexit(...)\n");
		SDL_Quit();
		return 1;
	}

	SDL_WM_SetCaption("Valtra Tractors","Valtra Tractors");
	Uint32 wflags = SDL_ANYFORMAT | SDL_SWSURFACE | SDL_HWACCEL | SDL_ASYNCBLIT;
	Uint32 fflags = SDL_ANYFORMAT | SDL_SWSURFACE | SDL_HWACCEL | SDL_ASYNCBLIT | SDL_FULLSCREEN;
	Uint32 flags = fflags;
	int players = 2;
	bool direct = false;
	for (int i = 1; i < argc; i++) {
		if (!strcmp(argv[i],"-f") || !strcmp(argv[i],"-F")) flags = fflags;
		else if (!strcmp(argv[i],"-w") || !strcmp(argv[i],"-W")) flags = wflags;
		else if (!strcmp(argv[i],"-d")) direct = true;
		else if (atoi(argv[i])>=0 && atoi(argv[i])<=4) players = atoi(argv[i]);
	}
	SDL_Surface *screen = SDL_SetVideoMode(800,600,0,flags);

	rects = (SDL_Rect*) malloc(sizeof(SDL_Rect)*arraySize);
	if (rects==NULL) {
		fprintf(stderr,"Out of memory\n");
		return 1;
	}


	srand(time(NULL));


	SDL_ShowCursor(SDL_DISABLE);
	Game *game;
	if (direct) {
		game = constructBoardGame(screen, 0);
	} else {
		game = constructWrapper(screen, 0);
	}
	game->startGame(players);
	game->setUpdatedRect(updatedRect);
	game->redraw();
//	SDL_Flip(screen);

	Uint32 starttime = SDL_GetTicks();
	int frames = 0;
	int totFrames = 0;
	Uint32 lasttime = 0;
	int totUpdated = 0;
	Uint32 lastredraw = 0;
	while (!game->finished()) {
		SDL_Event event;
		bool finished = false;
		while (SDL_PollEvent(&event)) {
			if (event.type==SDL_QUIT) {
				finished = true;
			} else if (event.type==SDL_VIDEOEXPOSE || event.type==SDL_SYSWMEVENT) {
				game->redraw();
				lastredraw = SDL_GetTicks();
			} else if ((event.type==SDL_KEYDOWN || event.type==SDL_KEYUP) && event.key.keysym.sym==SDLK_RETURN && event.key.keysym.mod&KMOD_ALT) {
				if (event.type==SDL_KEYDOWN) {
					if (flags&SDL_FULLSCREEN) flags = wflags;
					else flags = fflags;
					screen = SDL_SetVideoMode(screen->w,screen->h,0,flags);
					SDL_ShowCursor(SDL_DISABLE);
					game->redraw();
				}
			} else {
				game->handleEvent(event);
			}
		}
		if (finished) break;

/*		if (frames%150==0 || SDL_GetTicks()-lastredraw>3500) {
			game->redraw();
			lastredraw = SDL_GetTicks();
		}
*/
		Uint32 time = SDL_GetTicks()-starttime;
		game->update(time,screen);

		SDL_UpdateRects(screen,nUpdated,rects);
		totUpdated += nUpdated;
		nUpdated = 0;

		frames++;
		time = SDL_GetTicks()-starttime;
		if (time-lasttime>1000) {
#ifdef FPS
			printf("%d frames in %d msec (%d fps) average %.1f rects per frame\n",frames,time-lasttime,1000*frames/(time-lasttime),float(totUpdated)/frames);
#endif
			lasttime = time;
			totFrames += frames;
			totUpdated = 0;
			frames = 0;
		}

		SDL_Delay(5);
	}
	totFrames += frames;
#ifdef FPS
	printf("Average: %d frames in %d msec (%d fps)\n",totFrames,SDL_GetTicks()-starttime,1000*totFrames/(SDL_GetTicks()-starttime));
#endif

	delete game;

	free(rects);

	return 0;
}

